#!/usr/bin/env bats

. test/helpers.sh
mas () { . $BORK_SOURCE_DIR/types/mas.sh $*; }

setup () {
    respond_to "uname -s" "echo Darwin"
    respond_to "mas account" "echo 'user@example.com'"
    respond_to "mas list" "cat $fixtures/mas-list.txt"
    respond_to "mas outdated" "cat $fixtures/mas-outdated.txt"
}

@test "mas status: returns FAILED_PRECONDITION without mas exec" {
    respond_to "which mas" "return 1"
    run mas status 497799835 Xcode
    [ "$status" -eq $STATUS_FAILED_PRECONDITION ]
}

@test "mas status: returns FAILED_PRECONDITION when logged out" {
    respond_to "mas account" "return 1"
    respond_to "uname -r" "echo 20.0.0"
    run mas status 497799835 Xcode
    [ "$status" -eq $STATUS_FAILED_PRECONDITION ]
}

@test "mas status: does not return FAILED_PRECONDITION when mas account fails on macOS >= 12" {
    respond_to "mas account" "return 1"
    respond_to "uname -r" "echo 21.4.0"
    run mas status 497799835 Xcode
    [ "$status" -eq $STATUS_OK ]
}

@test "mas status: does not return FAILED_PRECONDITION when mas account fails on macOS >= 13" {
    respond_to "mas account" "return 1"
    respond_to "uname -r" "echo 22.2.0"
    run mas status 497799835 Xcode
    [ "$status" -eq $STATUS_OK ]
}

@test "mas status: returns MISSING when app not installed" {
    run mas status 477670270 2Do
    [ "$status" -eq $STATUS_MISSING ]
}

@test "mas status: returns OUTDATED when app upgrade pending" {
    run mas status 458034879 Dash
    [ "$status" -eq $STATUS_OUTDATED ]
}

@test "mas status: returns OK when app installed and up-to-date" {
    run mas status 497799835 Xcode
    [ "$status" -eq $STATUS_OK ]
}

@test "mas install: performs install" {
    run mas install 477670270 2Do
    [ "$status" -eq 0 ]
    run baked_output
    [ "$output" = 'mas install 477670270' ]
}

@test "mas upgrade: performs upgrade" {
    run mas upgrade 458034879 Dash
    [ "$status" -eq 0 ]
    run baked_output
    [ "$output" = 'mas upgrade' ]
}

@test "mas inspect: returns FAILED_PRECONDITION without mas exec" {
    respond_to "which mas" "return 1"
    run mas inspect
    [ "$status" -eq $STATUS_FAILED_PRECONDITION ]
}

@test "mas inspect: returns OK if preconditions met" {
    run mas inspect
    [ "$status" -eq $STATUS_OK ]
}

@test "mas remove: performs uninstall" {
    run mas remove 477670270 2Do
    [ "$status" -eq 0 ]
    run baked_output
    [ "$output" = 'mas uninstall 477670270' ]
}
